@@ -86,11 +86,12 @@ def get_hierarchy_label(batch_labels, hierarchy_level):
 def map_labels(label_map, labels):
     labels = to_numpy(labels)
     if labels.ndim == 2:
+        new_labels = np.zeros(labels.shape, dtype=int)
         for h in range(labels.shape[1]):
-            labels[:, h] = label_map(labels[:, h], h)
+            new_labels[:, h] = label_map(labels[:, h], h)
     else:
-        labels = label_map(labels, 0)
-    return labels
+        new_labels = label_map(labels, 0)
+    return new_labels
 
 
 def process_label(labels, hierarchy_level, label_map):
@@ -235,7 +236,9 @@ def make_label_to_rank_dict(label_set):
     Returns:
         A dictionary mapping each label to its numeric rank in the original set
     """
-    ranked = scipy.stats.rankdata(label_set) - 1
+    if len(set(label_set)) != len(label_set):
+        raise ValueError("label set must not have duplicates")
+    ranked = scipy.stats.rankdata(label_set).astype(int) - 1
     return {k: v for k, v in zip(label_set, ranked)}
 
 
@@ -406,13 +409,13 @@ def return_input(x):
 
 
 def check_shapes(embeddings, labels):
-    if embeddings.shape[0] != labels.shape[0]:
+    if labels is not None and embeddings.shape[0] != labels.shape[0]:
         raise ValueError("Number of embeddings must equal number of labels")
     if embeddings.ndim != 2:
         raise ValueError(
             "embeddings must be a 2D tensor of shape (batch_size, embedding_size)"
         )
-    if labels.ndim != 1:
+    if labels is not None and labels.ndim != 1:
         raise ValueError("labels must be a 1D tensor of shape (batch_size,)")
 
 
@@ -495,7 +498,8 @@ def set_ref_emb(embeddings, labels, ref_emb, ref_labels):
     if ref_emb is not None:
         if not torch.is_tensor(ref_labels):
             TypeError("if ref_emb is given, then ref_labels must also be given")
-        ref_labels = to_device(ref_labels, ref_emb)
+        if ref_labels is not None:
+            ref_labels = to_device(ref_labels, ref_emb)
     else:
         ref_emb, ref_labels = embeddings, labels
     check_shapes(ref_emb, ref_labels)
@@ -512,6 +516,16 @@ def indices_tuple_not_supported(indices_tuple):
         raise ValueError("indices_tuple is not supported for this loss function")
 
 
+def labels_required(labels):
+    if labels is None:
+        raise ValueError("labels are required for this loss function")
+
+
+def labels_or_indices_tuple_required(labels, indices_tuple):
+    if labels is None and indices_tuple is None:
+        raise ValueError("labels and indices_tuple cannot both be None")
+
+
 def concatenate_indices_tuples(it1, it2):
     return tuple([torch.cat([x, to_device(y, x)], dim=0) for x, y in zip(it1, it2)])
 
